/*
    File:       ATAExtras.h
 
    Contains:   Definitions required to implement an AIM.
 
    Version:    Technology: System 7.5
                Release:    Universal Interfaces 3.x
 
    Copyright:   1995-1998 by Apple Computer, Inc., all rights reserved
 
    Bugs?:      For bug reports, consult the following page on
                the World Wide Web:
 
                    http://developer.apple.com/bugreporter/
 
                This file is based on "ATAPriv.i,40" with non-AIM related
                stuff removed -- Quinn, 9 Jun 1999
*/
#ifndef __ATAEXTRAS__
#define __ATAEXTRAS__

#ifndef __OSUTILS__
#include <OSUtils.h>
#endif

#ifndef __ATA__
#include <ATA.h>
#endif

#ifndef __KERNEL__
#include <Kernel.h>
#endif

#ifndef __NAMEREGISTRY__
#include <NameRegistry.h>
#endif

#ifndef __CODEFRAGMENTS__
#include <CodeFragments.h>
#endif

#ifndef __MACTYPES__
#include <MacTypes.h>
#endif




#if PRAGMA_ONCE
#pragma once
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_IMPORT
#pragma import on
#endif

#if PRAGMA_STRUCT_ALIGN
    #pragma options align=mac68k
#elif PRAGMA_STRUCT_PACKPUSH
    #pragma pack(push, 2)
#elif PRAGMA_STRUCT_PACK
    #pragma pack(2)
#endif

/* The Function codes */
enum {
    kATAFnNOP                   = 0x00,                         /* No Operation*/
    kATAFnExecIO                = 0x01,                         /* Execute ATA I/O*/
    kATAFnBusInquiry            = 0x02,                         /* Bus Inquiry*/
    kATAFnQRelease              = 0x03,                         /* I/O Queue Release*/
    kATAFnCmd                   = 0x04,                         /* ATA Command*/
    kATAFnAbort                 = 0x05,                         /* Abort command*/
    kATAFnBusReset              = 0x06,                         /* Reset ATA bus*/
    kATAFnRegAccess             = 0x07,                         /* Register Access*/
    kATAFnDriveIdentify         = 0x08,                         /* Drive Identify*/
    kATAPIFnExecIO              = 0x09,                         /* ATAPI I/O*/
    kATAPIFnCmd                 = 0x0A,                         /* ATAPI Command*/
    kATAFnGetDriveConfig        = 0x0B,                         /* Get Drive specific configuration*/
    kATAFnSetDriveConfig        = 0x0C,                         /* Set Drive specific configuration*/
    kATAFnKillIO                = 0x0D                          /* Kill any current interaction with bus*/
};

/* New ATA Manager Function Code Definition*/
enum {
    kATAMgrAddATABus            = 0x93,                         /* Add an AIM to ATA Manager*/
    kATAMgrRemoveATABus         = 0x94                          /* Remove an AIM from ATA Manager*/
};

/* The state to put the device light in used in DeviceLight plugin export*/
enum {
    kATADeviceLightOff          = 0x00,                         /* Turn light off*/
    kATADeviceLightOn           = 0x01                          /* Turn light on*/
};

/* The state to put the device lock in used in DeviceLock plugin export*/
enum {
    kATADeviceUnlock            = 0x00,                         /* unlock*/
    kATADeviceLock              = 0x01                          /* lock*/
};

/* add bus flags*/
/* these apply to the flags field of the ataAddATABus structure  */
enum {
    mATANoDMAOnBus              = 0x80
};

/*====================================================================================*/

struct ATADataObject {
    UInt8 *                         ioBuf;                      /* if SG enabled then this points to ATASGList */
    UInt32                          Count;                      /* if SG enabled then count of SG Lists else the size */
};
typedef struct ATADataObject            ATADataObject;

struct ATAResult {
    OSStatus                        ataResult;                  /* the final result see Error Codes */
    SInt8                           ataStatusRegister;          /* the status register */
    SInt8                           ataErrorRegister;           /* the error register */
    UInt32                          actualXferCount;            /* in case of I/O the actual size of Xfer */
    ataTaskFile *                   TaskFile;                   /* if "mATAFlagTFRead" enabled */
};
typedef struct ATAResult                ATAResult;

struct ATADevInfo {
    UInt8                           devType;
    SInt8                           devID;
};
typedef struct ATADevInfo               ATADevInfo;
/* ATA Device ID codes to be used with devID field above*/
enum {
    kATAInvalidDeviceID         = -1,
    kATADevice0DeviceID         = 0,
    kATADevice1DeviceID         = 1
};


struct ATAInitInfo {
    UInt32                          busID;                      /* --> busID assigned by the Manager*/
    ATADevInfo                      FirstDevice;                /* <-- Type and ID of first device*/
    ATADevInfo                      SecondDevice;               /* <-- Type and ID of second device*/
    RegEntryIDPtr                   aimRegEntry;                /* --> Name Registry entry for this controller*/
    UInt32                          refCon;                     /* <-- refCon to be associated with this instantiation of the AIM*/
};
typedef struct ATAInitInfo              ATAInitInfo;

struct ATADiagResult {
    UInt16                          ataRegMask;                 /* the registers to access */
    OSStatus                        ataResult;                  /* the final result see Error Codes */
    UInt16                          ataDataReg;                 /* 16 bit access only */
    UInt8                           ataTFFeatures;              /* Error(R)  or Feature(W) register */
    UInt8                           ataTFCount;                 /* Sector Count register */
    UInt8                           ataTFSector;                /* Sector number register */
    UInt8                           ataTFCylinderLo;            /* Cylinder low bytes  */
    UInt8                           ataTFCylinderHi;            /* Cylinder High bytes  */
    UInt8                           ataTFSDH;                   /* SDH register */
    UInt8                           ataTFCommand;               /* status(R) or command(W) register */
    UInt8                           ataAltStatDevCnt;           /* AltStatus(R) or Dev Cntrl(W) register */
};
typedef struct ATADiagResult            ATADiagResult;

struct ATABusInfo {
    UInt8                           ataPIOModes;                /* PIO modes supported (bit-significant)*/
    UInt8                           ataSingleDMAModes;          /* <--: Single Word DMA modes supported (b-sig) */
    UInt8                           ataMultiDMAModes;           /* <--: Multiword DMA modes supported (b-sig)*/
    UInt8                           ataUltraDMAModes;           /* <--: Ultra DMA modes supported (b-sig)*/
    UInt32                          ataIOPBsize0;               /* the current size of the Transfer for device 0 *|*/
    UInt32                          ataIOPBsize1;               /* the current size of the Transfer for device 1 *|*/
    SInt8                           ataContrlType[16];          /* the controller signature *|*/
    NumVersion                      ataHBAversion;              /* version number of the AIM*/
    UInt32                          reserved3;
};
typedef struct ATABusInfo               ATABusInfo;

struct ATADevConfig {
    SInt32                          ataConfigSetting;           /* <->: Configuration setting*/
                                                                /*    Bits 3 - 0: Reserved*/
                                                                /*    Bit 4: Reserved (allowLBAAccess)*/
                                                                /*    Bit 5: Reserved (allowRWMultiple)*/
                                                                /*    Bit 6: ATAPIpacketDRQ*/
                                                                /*      1 = Check for Interrupt DRQ on ATAPI command packet DRQ*/
                                                                /*      0 = Default: Check only for the assertion of command packet DRQ*/
                                                                /*    Bits 31 - 7: Reserved*/
    UInt8                           ataPIOSpeedMode;            /* <->: Device access speed in PIO Mode*/
    UInt8                           reserved;                   /*  padding*/
    UInt16                          atapcValid;                 /* <->: Set when pcXXX fields are valid (atapcAccessMode - atapcConfigIndex)*/
                                                                /*      bit 0 - atapcAccessMode field valid, when set*/
                                                                /*      bit 1 - atapcVcc field valid, when set*/
                                                                /*      bit 2 - atapcVpp1 field valid, when set*/
                                                                /*      bit 3 - atapcVpp2 field valid, when set*/
                                                                /*      bit 4 - atapcStatus field valid, when set*/
                                                                /*      bit 5 - atapcPin field valid, when set*/
                                                                /*      bit 6 - atapcCopy field valid, when set*/
                                                                /*      bit 7 - atapcConfigIndex field valid, when set*/
                                                                /*      bits 14-8 - Reserved*/
                                                                /*      bit 15 - device lock/unlock request (write only)*/
    UInt16                          ataRWMultipleCount;         /* Reserved for future (not supported yet)*/
    UInt16                          ataSectorsPerCylinder;      /* Reserved for future (not supported yet)*/
    UInt16                          ataHeads;                   /* Reserved for future (not supported yet)*/
    UInt16                          ataSectorsPerTrack;         /* Reserved for future (not supported yet)*/
    UInt16                          ataSocketNumber;            /* <--: Socket number used by the CardServices*/
                                                                /*      0xFF = socket number invalid (Not a CardServices device)*/
                                                                /*      other = socket number of the device*/
    UInt8                           ataSocketType;              /* <--: Specifies the socket type (get config only)*/
                                                                /*      00 = Unknown socket*/
                                                                /*      01 = Internal ATA bus*/
                                                                /*      02 = Media Bay*/
                                                                /*      03 = PCMCIA*/
    UInt8                           ataDeviceType;              /* <--: Specifies the device type (get config only)*/
                                                                /*      00 = Unknown device*/
                                                                /*      01 = standard ATA device (HD)*/
                                                                /*      02 = ATAPI device*/
                                                                /*      03 = PCMCIA ATA device*/
    UInt8                           atapcAccessMode;            /* <->: Access mode: Memory vs. I/O (PCMCIA only)*/
    UInt8                           atapcVcc;                   /* <->: Voltage in tenths of a volt (PCMCIA only)*/
    UInt8                           atapcVpp1;                  /* <->: Voltage in tenths of a volt (PCMCIA only)*/
    UInt8                           atapcVpp2;                  /* <->: Voltage in tenths of a volt (PCMCIA only)*/
    UInt8                           atapcStatus;                /* <->: Card Status register setting (PCMCIA only)*/
    UInt8                           atapcPin;                   /* <->: Card Pin register setting (PCMCIA only)*/
    UInt8                           atapcCopy;                  /* <->: Card Socket/Copy register setting (PCMCIA only)*/
    UInt8                           atapcConfigIndex;           /* <->: Card Option register setting (PCMCIA only)*/
    UInt8                           ataSingleDMASpeed;          /* <->: Single Word DMA Timing Class*/
    UInt8                           ataMultiDMASpeed;           /* <->: Multiple Word DMA Timing Class*/
    UInt16                          ataPIOCycleTime;            /* <->:Cycle time for PIO mode*/
    UInt16                          ataMultiCycleTime;          /* <->:Cycle time for Multiword DMA mode*/
    UInt8                           ataUltraDMASpeed;           /* <-> Ultra DMA timing class*/
    UInt8                           reserved2;                  /* reserved*/
    UInt16                          ataUltraCycleTime;          /* <-> Cycle time for Ultra DMA mode*/
    UInt16                          Reserved1[5];               /* Reserved for future*/
};
typedef struct ATADevConfig             ATADevConfig;
/* For 68k version of ATAMgrv4.0 ONLY */
/* connectionID is the pointer to globals used by the plugin */
/* MsgID is the pointer to the Sys 7.0 pb that is behind this request. */

struct ATAReqBlock {
                                                                /* Filled in by ATA Mgr v4.0*/
    UInt32                          connectionID;               /* the connection ID of the req block */
    UInt32                          MsgID;                      /* the ioTag value of this request */
    ATAResult *                     result;                     /* result of this operation */
    ATADiagResult *                 DiagResult;                 /* for diagnostics i.e. R(W) registers */
    ATABusInfo *                    busInfo;                    /* for bus info requests */
    ATADevConfig *                  devConfig;                  /* for device config requests */
    ATADataObject                   ioObject;                   /* The actual transfer data */
    ataTaskFile                     ataPBTaskFile;              /* the task file */
    ATAPICmdPacket                  packetCBD;                  /* For the ATAPI cmd packets */
    Duration                        Timeout;                    /* timeout for this request */
    UInt32                          BusID;                      /* family assigned bus ID */
    SInt8                           DevID;                      /* device ID, -1(bus), 0(master), 1(slave) */
    UInt8                           ataFunctionCode;
    UInt32                          AbortID;
    UInt32                          ataPBLogicalBlockSize;
    UInt32                          ataPBFlags;
    UInt32                          reserved;
                                                                /* Used internally by the AIM*/
    struct ATAReqBlock *            nextREQ;                    /* the next req on this list */
    OSStatus                        ataPBResult;
    UInt8                           ataPBErrorRegister;
    UInt8                           ataPBStatusRegister;
    UInt32                          ataPBactualXferCount;
    UInt32                          ataPBState;
    UInt32                          ataPBSemaphores;
    UInt8                           XferType;
    UInt8                           ataModeType;                /* tracks old ataPBVers, to do absolute or bitmap modes     */
    UInt8                           ataPBIOSpeed;               /* -->: I/O Timing Class            */
    UInt8                           reserved2;
    UInt16                          reserved3;
};
typedef struct ATAReqBlock              ATAReqBlock;
/* enum for ataModeType above, aligned with ataPBVers*/
enum {
    kATAModeAbsolute            = 2,
    kATAModeBitmap              = 3                             /* actually three or above*/
};

enum {
	kServiceCategoryATA			= 'ata-'						/* AIM 'ndrv' service category */
};

enum {
    kATAPluginVersion           = 0x00000001,
    kATAPluginCurrentVersion    = kATAPluginVersion
};

/* Typedefs for the AIM entry point pointers*/
typedef CALLBACK_API_C( OSStatus , ATAPluginInit )(ATAInitInfo *pb);
typedef CALLBACK_API_C( OSStatus , ATAPluginClose )(UInt32 refCon, RegEntryIDPtr aimRegEntry);
typedef CALLBACK_API_C( void , ATAPluginAction )(UInt32 refCon, ATAReqBlock *pb);
typedef CALLBACK_API_C( void , ATAPluginHandleBusEvent )(UInt32 refCon, UInt32 aimData);
typedef CALLBACK_API_C( Boolean , ATAPluginPoll )(UInt32 refCon, UInt32 interruptLevel, UInt32 *aimData);
typedef CALLBACK_API_C( void , ATAPluginEjectDevice )(UInt32 refCon);
typedef CALLBACK_API_C( void , ATAPluginDeviceLight )(UInt32 refCon, UInt32 whichDevice, UInt32 lightState);
typedef CALLBACK_API_C( void , ATAPluginDeviceLock )(UInt32 refCon, UInt32 whichDevice, UInt32 lockState);
typedef CALLBACK_API_C( void , ATAPluginSuspend )(UInt32 refCon);
typedef CALLBACK_API_C( void , ATAPluginResume )(UInt32 refCon);

struct ATAPluginHeader {
    NumVersion                      headerVersion;
    NumVersion                      dispatchVersion;
    UInt32                          reservedA;
    UInt32                          reservedB;
};
typedef struct ATAPluginHeader          ATAPluginHeader;

/*  your AIM must export this structure under the name "ThePluginDispatchTable"  */

struct ATAPluginDispatchTable {
    ATAPluginHeader                 header;
    ATAPluginInit                   init;
    ATAPluginClose                  close;
    ATAPluginAction                 action;
    ATAPluginHandleBusEvent         busEvent;
    ATAPluginPoll                   poll;
    ATAPluginEjectDevice            eject;
    ATAPluginDeviceLight            light;
    ATAPluginDeviceLock             lock;
    ATAPluginSuspend                suspend;
    ATAPluginResume                 resume;
};
typedef struct ATAPluginDispatchTable   ATAPluginDispatchTable;

/* Parameter block structure for adding an ATA bus*/

struct ataAddATABus {
                                                                /* Start of cloned common header ataPBHdr*/
    ataPBHeader *                   ataPBLink;                  /* a pointer to the next entry in the queue */
    UInt16                          ataPBQType;                 /* type byte for safety check*/
    UInt8                           ataPBVers;                  /* -->: parameter block version number; Must be 0x01*/
    UInt8                           ataPBReserved;              /* Reserved                                     */
    Ptr                             ataPBReserved2;             /* Reserved                                     */
    ProcPtr                         ataPBCallbackPtr;           /* -->: Completion Routine Pointer  */
    OSErr                           ataPBResult;                /* <--: Returned result             */
    UInt8                           ataPBFunctionCode;          /* -->: Manager Function Code */
    UInt8                           ataPBIOSpeed;               /* -->: I/O Timing Class            */
    UInt16                          ataPBFlags;                 /* -->: Various control options */
    SInt16                          ataPBReserved3;             /* Reserved                                     */
    UInt32                          ataPBDeviceID;              /* -->: Device identifier           */
    UInt32                          ataPBTimeOut;               /* -->: Transaction timeout value   in msec */
    Ptr                             ataPBClientPtr1;            /* Client's storage Ptr 1   */
    Ptr                             ataPBClientPtr2;            /* Client's storage Ptr 2   */
    UInt16                          ataPBState;                 /* Reserved for Manager; Initialize to 0 */
    UInt16                          ataPBSemaphores;            /* Used internally by the manager*/
    SInt32                          ataPBReserved4;             /* Reserved                                     */
                                                                /* End of cloned common header ataPBHdr*/
    RegEntryIDPtr                   ataNameRegEntry;            /* -->: Pointer to name registry data*/
    CFragConnectionID               connID;                     /* -->: Connection to use. Should be zero.*/
    UInt32                          busID;                      /* <--: bus ID*/
    UInt8                           flags;                      /* -->: add bus flags*/
    UInt8                           socketType;                 /* -->: Socket type enum from ATA.i*/
    Ptr                             iconData;                   /* -->: Ptr to 256 bytes of icon data (nil is OK)*/
    Ptr                             stringData;                 /* -->: C String defining location of device. (nil is OK)*/
};
typedef struct ataAddATABus             ataAddATABus;
/* Parameter block structure for adding an ATA bus*/

struct ataRemoveATABus {
                                                                /* Start of cloned common header ataPBHdr*/
    ataPBHeader *                   ataPBLink;                  /* a pointer to the next entry in the queue */
    UInt16                          ataPBQType;                 /* type byte for safety check*/
    UInt8                           ataPBVers;                  /* -->: parameter block version number; Must be 0x01*/
    UInt8                           ataPBReserved;              /* Reserved                                     */
    Ptr                             ataPBReserved2;             /* Reserved                                     */
    ProcPtr                         ataPBCallbackPtr;           /* -->: Completion Routine Pointer  */
    OSErr                           ataPBResult;                /* <--: Returned result             */
    UInt8                           ataPBFunctionCode;          /* -->: Manager Function Code */
    UInt8                           ataPBIOSpeed;               /* -->: I/O Timing Class            */
    UInt16                          ataPBFlags;                 /* -->: Various control options */
    SInt16                          ataPBReserved3;             /* Reserved                                     */
    UInt32                          ataPBDeviceID;              /* -->: Device identifier           */
    UInt32                          ataPBTimeOut;               /* -->: Transaction timeout value   in msec */
    Ptr                             ataPBClientPtr1;            /* Client's storage Ptr 1   */
    Ptr                             ataPBClientPtr2;            /* Client's storage Ptr 2   */
    UInt16                          ataPBState;                 /* Reserved for Manager; Initialize to 0 */
    UInt16                          ataPBSemaphores;            /* Used internally by the manager*/
    SInt32                          ataPBReserved4;             /* Reserved                                     */
                                                                /* End of cloned common header ataPBHdr*/
    UInt32                          busID;                      /* -->: bus ID to remove                        */
    RegEntryIDPtr                   ataNameRegEntry;            /* -->: Pointer to name registry data       (Used by native ATA. above to be obsoleted)*/
};
typedef struct ataRemoveATABus          ataRemoveATABus;

/* Callbacks into the System 7 ATA Manager*/
EXTERN_API_C( void )
ATAFamIODone                    (ATAReqBlock *          theReq,
                                 OSStatus               result);

EXTERN_API_C( void )
ATAFamBusEventForAIM            (UInt32                 busID,
                                 UInt32                 aimData);

/* property string to be used for ATA device type*/
#define kATADeviceType                  "ata" 

#if PRAGMA_STRUCT_ALIGN
    #pragma options align=reset
#elif PRAGMA_STRUCT_PACKPUSH
    #pragma pack(pop)
#elif PRAGMA_STRUCT_PACK
    #pragma pack()
#endif

#ifdef PRAGMA_IMPORT_OFF
#pragma import off
#elif PRAGMA_IMPORT
#pragma import reset
#endif

#ifdef __cplusplus
}
#endif

#endif /* __ATAEXTRAS__ */
